from pathlib import Path
import os
from glob import glob
from urllib.parse import urlparse


class Util:
    @staticmethod
    def relative_files_from_dir(directory: str) -> dict:
        abspath = os.path.abspath(directory)
        parent = Path(abspath).parent
        glob_string = f"{abspath}/**/*"
        paths = glob(glob_string, recursive=True)

        results = {}
        for path in paths:
            if os.path.isfile(path):
                relative_path = Util._remove_prefix(path, str(parent) + "/")
                results[path] = relative_path
        return results

    @staticmethod
    def get_bucket_and_prefix_from_url(url: str):
        o = urlparse(url)
        return o.scheme, o.netloc, o.path.lstrip("/")

    def _remove_prefix(text: str, prefix: str) -> str:
        if text.startswith(prefix):
            return text[len(prefix) :]
        return text



import click
import sys
from .core import Core
from .uploader import Uploader
from .downloader import Downloader
from . import constants
from .logger import get_logger


_global_options = [
    click.option("--url", "-u", type=str, required=True),
    click.option("--debug/--no-debug", default=False, help="enable debug logging"),
]


def global_options(func):
    for option in reversed(_global_options):
        func = option(func)
    return func


class Mash(object):
    pass


@click.group(invoke_without_command=True)
@click.option(
    "--version/--no-version", "-v", default=False, help="show version. (default: False)"
)
@click.option("--aws-profile", default=None, help="aws profile name")
@click.pass_context
def cli(ctx, version, aws_profile):
    ctx.obj = Mash()
    ctx.obj.aws_profile = aws_profile
    if version:
        print(constants.VERSION)
        sys.exit()

    if ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())


@cli.command(help="list local files.")
@global_options
@click.option("--download/--no-download", "-d", default=False, help="download files")
@click.pass_context
def list_local(ctx, url: str, debug: str, download):
    s3local = Downloader(url=url, logger=get_logger(debug=debug))
    paths = s3local.list_local_path(download=download)
    for path in paths:
        print(path)


@cli.command(help="list local files.")
@global_options
@click.pass_context
def list(ctx, url, debug):
    s3local = Downloader(url=url, logger=get_logger(debug=debug))
    paths = s3local.list_download_path()
    for path in paths:
        print(path)


@cli.command(help="download file by s3.")
@global_options
@click.option("--skip-exist/--no-skip-exist", default=True, help="download files")
@click.pass_context
def download(ctx, url: str, debug: str, skip_exist: bool):
    s3local = Downloader(url=url, logger=get_logger(debug=debug))
    s3local.download(skip_exist=skip_exist)


@cli.command(help="delete file by s3.")
@global_options
@click.pass_context
def delete(ctx, url: str, debug: bool):
    s3local = Core(url=url, logger=get_logger(debug=debug))
    s3local.delete()


@cli.command(help="upload file to s3.")
@global_options
@click.option("--source", "-s", type=str, required=True)
@click.option("--skip-exist/--no-skip-exist", default=True, help="download files")
@click.pass_context
def upload(ctx, url: str, debug: str, source: str, skip_exist: bool):
    s3local = Uploader(url=url, logger=get_logger(debug=debug))
    s3local.upload(source_path=source, skip_exist=skip_exist)


def main():
    cli(obj={})



from boto3.session import Session
import os
import shutil

from .logger import get_logger
from .util import Util
from . import constants


class Core:
    def __init__(
        self,
        url: str,
        root: str = (os.environ.get("S3LOCAL_ROOT") or constants.DEFAULT_S3LOCAL_ROOT),
        aws_profile: str = None,
        logger=get_logger(),
    ):
        scheme, bucket_name, prefix = Util.get_bucket_and_prefix_from_url(url)
        self.bucket_name = bucket_name
        self.prefix = prefix
        self.recursive = True if prefix.endswith("/") else False

        self.root = os.path.join(root, scheme, bucket_name)
        os.makedirs(self.root, exist_ok=True)
        self.local_path = os.path.join(self.root, prefix)
        self.bucket = (
            (Session() if aws_profile is None else Session(profile_name=aws_profile))
            .resource("s3")
            .Bucket(bucket_name)
        )
        self.logger = logger
        self.download_paths = []

    def get_local_root(self) -> str:
        return self.root

    def delete(self):
        self.logger.info(f"delete => s3://{self.bucket_name}/{self.prefix}")
        self.logger.debug(f"delete => {self.local_path}")
        if self.recursive:
            shutil.rmtree(self.local_path)
            self.bucket.objects.filter(
                Prefix=self.prefix,
            ).delete()
        else:
            os.remove(self.local_path)
            self.bucket.delete_object(
                Key=self.prefix,
            )

    def exists_key(self, key: str) -> bool:
        keys = self.bucket.objects.filter(Prefix=key)
        return key in [k.key for k in keys]



import os
import glob
from .core import Core


class Downloader(Core):
    def download_file(
        self,
        key: str,
        dryrun: bool = False,
        skip_exist: bool = True,
        dst_path: str = None,
    ):
        dst_path = dst_path or f"{self.root}/{key}"
        dst_dir_path = os.path.dirname(dst_path)
        s3_url = f"s3://{self.bucket_name}/{key}"
        os.makedirs(dst_dir_path, exist_ok=True)
        if os.path.exists(dst_path) and skip_exist:
            self.logger.info(f"skip already exists in local: {s3_url} > {dst_path}")
        else:
            self.logger.debug(f"Copying: {s3_url} > {dst_path}")
            if not dryrun:
                self.bucket.download_file(key, dst_path)
        self.download_paths.append(dst_path)

    def download(
        self, dryrun: bool = False, skip_exist: bool = True, dst_path: str = None
    ):
        if self.recursive:
            objects = self.bucket.objects.filter(
                Prefix=self.prefix,
            )
            for key in [o.key for o in objects]:
                dst_path = f"{dst_path}/{os.path.basename(key)}" if dst_path else None
                self.download_file(
                    key, dryrun, skip_exist=skip_exist, dst_path=dst_path
                )
        else:
            self.download_file(
                self.prefix, dryrun, skip_exist=skip_exist, dst_path=dst_path
            )

    def list_download_path(self):
        self.download()
        return self.download_paths

    def list_local_path(self, download: bool = False):
        if download:
            self.download()

        # Search cache only without accessing s3
        if self.recursive:
            glob_string = f"{self.local_path}**/*"
            paths = glob.glob(glob_string, recursive=True)
            paths = [x for x in paths if os.path.isfile(x)]  # file only
        else:
            paths = [self.local_path]

        return paths

    def get_local_path(self, download=False):
        if download:
            self.download()
        return self.local_path



import os
import shutil

from .util import Util
from .core import Core


class Uploader(Core):
    def upload(self, source_path: str, skip_exist: bool = True):
        basename = os.path.basename(source_path)

        if os.path.isfile(source_path):
            if self.prefix.endswith("/"):
                key = f"{self.prefix}{basename}"
            else:
                # change basename
                key = self.prefix

            # Check if the key exists
            if skip_exist and self.exists_key(key=key):
                self.logger.info(f"skip upload {source_path} > {key}")
            else:
                self.upload_file(source_path, key)
        elif os.path.isdir(source_path):
            if self.recursive:
                objects = self.bucket.objects.filter(
                    Prefix=self.prefix,
                )
                already_upload_keys = [o.key for o in objects]
                pair = Util.relative_files_from_dir(directory=source_path)
                for abspath, relative_path in pair.items():
                    upload_key = f"{self.prefix}{relative_path}"
                    if upload_key in already_upload_keys:
                        self.logger.info(f"skip upload {upload_key}")
                    else:
                        self.upload_file(abspath, upload_key)
            else:
                raise "not implement"

    def upload_file(self, local_path: str, key: str):
        # copy local
        dst_path = os.path.join(self.root, key)
        self.logger.debug(f"Copying to local: {local_path} => {dst_path}")
        dst_dir_path = os.path.dirname(dst_path)
        os.makedirs(dst_dir_path, exist_ok=True)
        shutil.copyfile(local_path, dst_path)

        # copy s3
        self.logger.info(
            f"Copying to s3: {local_path} => s3://{self.bucket_name}/{key}"
        )
        self.bucket.upload_file(local_path, key)
